/*************************************************
                                                
   Programm: DEMO1                              
                                                
                                                
   Version:  1.0                                
   Datum:    24.06.2004                         

    Inhalte:
     - Interrupt-Programmierung
     - LC-Display
     - Tastaturabfrage
     - Analog-Digital-Wandler    

    Im Make-Wizard muss die Datei display.c     
    mit "ADD Source File..." hinzugefgt        
    werden.                                     
                                                
                                                
*************************************************/
#include <reg51.h>
#include <irq52.h>
#include <stdio.h>


/*          Hardware-Interface:          */

//   Tastatur
#define Key_1 P3_B5
#define Key_2 P3_B4
#define Key_3 P3_B3
#define Key_4 P3_B2


//   Analog-Digital-Umsetzer
#define AD_CS P1_B0

//   SPI - Schnittstelle
near unsigned char SPDAT @ 0xC5;            // Daten-
near unsigned char SPSTA @ 0xC4;            // Status-
near unsigned char SPCON @ 0xC3;            // Kontrollregister


//   Globale Variablen

near unsigned char Wartezeit = 5;

// Prototypen
int lcd_printf(far char* pfmt, ...);

/*          Zeitgeber          */
IRQ_VECTOR(timer0int,TIMER0)
void timer0int (void) interrupt  
{

  TR0 = 0;
  TH0 = 0xB1;                                   // set timer period: 20ms
  TL0 = 0xDF;
  TR0 = 1;

  Wartezeit--;
}


// setup the timer 0 interrupt 
void Timer_Start(void)
{
  TH0 = 0xB1;                                   // set timer period: 20ms
  TL0 = 0xDF;
  TMOD = TMOD | 0x01;                           // select mode 1            
  ET0 = 1;                                      // enable timer 0 interrupt 
  EA  = 1;                                      // global interrupt enable 
  TR0 = 1;     // Timer starten
}


unsigned char ADU(void)
{
  AD_CS = 0;                    // Baustein aktivieren
  SPCON = 0xF2;                 // Init SPI-Bus
  SPDAT = 0;                    // Dummy-Schreiben, um zu lesen
  while ( (SPSTA&0x80)==0 );    // Warten bis Lesevorgang abgeschlossen
  AD_CS = 1;
  return(SPDAT);
}

void Lauflicht(void)
{
  unsigned char temp;
  
  temp = P0^0xFF;
  temp = temp << 1;
  if (temp == 0) temp = 1;
  P0   = temp^0xFF;
  Wartezeit = 12;
}




/*          Hauptprogramm          */
void main(void)
{

 unsigned char tastatur;
 
 Timer_Start();
 lcd_printf("\a");   // Init Display
 lcd_printf("\fMExS 1.1       W");  // Startmeldung

 
 while(1)
     {
       
       if (Wartezeit==0) Lauflicht();

       tastatur = (P3 ^ 0xFF) >> 2;
       lcd_printf("\nADU: %3d Key: %2d",ADU(),tastatur);

      };

}


