/*************************************************
*                                                *
*   Modul zur Steuerung eines LC-Displays        *
*                                                *
*             *
*   Die Steuerung erfolgt ber Steuerzeichen     *
*   mittels lcd_printf().                        *
*                                                *
*   \a    Initialisierung                        *
*   \r    Display lschen, Cursor Home           *
*   \f    Cursor in erste  Zeile setzen          *
*   \n    Cursor in zweite Zeile setzen          *
*                                                *
*                                                *
*   Esc. Ziffer Cursor setzen                    *
*   0x1B 0x40 = Zeile2 Spalte 1                  *
*                                                *
*                                                *
*   Weitere Steuerzeichen knnen in putchar()    *
*   implementiert werden.                        *
*                                                *
*                                                *
*   Version:  1.0                                *
*   Datum:    17.07.2003                         *
*------------------------------------------------*
*   Version:  1.1                                *
*   Datum:    01.07.2004                         *
*   Anpassung an uC51 (www.wickenhaeuser.de)     *
*                                                *
*                                                *
*  DISPLAY.C beim Make-Wizard mit "Add Source.." *
*  in das Projekt einbinden. Im Hauptprogramm    *
*  muss lcd_printf() deklariert werden:          *
*                                                *
*   int lcd_printf(far char* pfmt, ...);         *
*                                                *
*                                                *
*************************************************/

#include <reg51.h>
#include <stdio.h>
#include <stdarg.h>

//   LC-Display
#define LCD_DATA  P2
#define LCD_E     P1_B2
#define LCD_RS    P1_B3
#define LCD_RW    P1_B4

near unsigned char Escape=0;    // Flag fr erweiterte Displaysteuerung

/*     Ansteuerung des LC-Display     */
void wait(unsigned short zeit) reentrant
{
  while (zeit >0) zeit--;
}

// Initialisierung lt. Hitachi-Datenbuch
void LCD_Init(void) reentrant
{
  wait(600); // Power-On Reset dauert 15ms
  wait(600);  
  LCD_RW = 0;
  LCD_RS = 0;
  LCD_E = 1;
  LCD_DATA = 0x30;
  LCD_E = 0;
  wait(500);
  LCD_E = 1;
  LCD_DATA = 0x30;
  LCD_E = 0;
  wait(100);
  LCD_E = 1;
  LCD_DATA = 0x30;
  LCD_E = 0;
  wait(50);
  LCD_E = 1;
  LCD_DATA = 0x30;
  LCD_E = 0;
  wait(50);
  LCD_E = 1;
  LCD_DATA = 0x38;    // 8bit, 2 Zeilen, 5x7 Font
  LCD_E = 0;
  wait(10);
  LCD_E = 1;
  LCD_DATA = 0x0C;    // Display ON
  LCD_E = 0;
  wait(10);
  LCD_E = 1;
  LCD_DATA = 0x01;    // Clear Display
  LCD_E = 0;
  wait(100);
  LCD_E = 1;
  LCD_DATA = 0x06;    // Increment +1, shift 0
  LCD_E = 0;
  wait(10);
}

// Einzelnes Zeichen ausgeben
void LCD_Write(char c) reentrant
{
  LCD_RS = 1;
  LCD_E = 1;
  LCD_DATA = c;
  LCD_E = 0;
 
}

// Display lschen
void LCD_Clear(void) reentrant
{
  LCD_RS = 0;
  LCD_E = 1;
  LCD_DATA = 0x01;
  LCD_E = 0;
  wait(500);
}

// Cursorposition setzen
void LCD_Goto(char c) reentrant
{
  LCD_RS = 0;
  LCD_E = 1;
  LCD_DATA = 0x80 | c;
  LCD_E = 0;
  wait(100);
}

// Zeichenausgabe auf LC-Display.
void lcd_putc (uchar c) reentrant
{
  if(Escape==0)
   {
     switch(c)
      {
        case 0x07: LCD_Init();  // Code \a
                   break;
        case 0x0D: LCD_Clear();    // Code \r
                   break;
        case 0x0C: LCD_Goto(0x00);   // Code \f
                   break;
        case 0x0A: LCD_Goto(0x40);   // Code \n
                   break;
        case 0x1B: Escape=1;  // Display-Steuercodes
                   break;
        default:   LCD_Write(c);  // Zeichenausgabe
      }

   }
  else
   {
      LCD_Goto(c);
      Escape=0;
   }
}

// This definition is not to be seen by default, because it needs <stdarg.h>
// So we include it manually...
extern int _doprnt( void (*iofunk)(uchar) reentrant, far char* pfmt, va_list (fap));

void lcd_printf(far char* pfmt, ...){
        va_list(ap);
        va_start(ap,pfmt);
        _doprnt(lcd_putc , pfmt, ap);  // And send it to LCD
}




